@@ -68,6 +68,7 @@
 
 #define GST_DISABLE_MINIOBJECT_INLINE_FUNCTIONS
 #include "gst_private.h"
+#include "gstidstr-private.h"
 #include <gst/gst.h>
 #include <gobject/gvaluecollector.h>
 
@@ -284,6 +285,42 @@ gst_caps_new_any (void)
   return caps;
 }
 
+/**
+ * gst_caps_new_id_str_empty_simple:
+ * @media_type: the media type of the structure
+ *
+ * Creates a new #GstCaps that contains one #GstStructure with name
+ * @media_type.
+ *
+ * Returns: (transfer full): the new #GstCaps
+ *
+ * Since: 1.26
+ */
+GstCaps *
+gst_caps_new_id_str_empty_simple (const GstIdStr * media_type)
+{
+  GstCaps *caps;
+  GstStructure *structure;
+
+  caps = gst_caps_new_empty ();
+  if (gst_id_str_is_equal_to_str (media_type, "ANY")) {
+    g_warning
+        ("media_type should not be ANY. Please consider using `gst_caps_new_any` or `gst_caps_from_string`.");
+  }
+  if (gst_id_str_get_len (media_type) == 0
+      || gst_id_str_is_equal_to_str (media_type, "EMPTY")
+      || gst_id_str_is_equal_to_str (media_type, "NONE")) {
+    g_warning
+        ("media_type should not be `%s`. Please consider using `gst_caps_new_empty` or `gst_caps_from_string`.",
+        gst_id_str_as_str (media_type));
+  }
+  structure = gst_structure_new_id_str_empty (media_type);
+  if (structure)
+    gst_caps_append_structure_unchecked (caps, structure, NULL);
+
+  return caps;
+}
+
 /**
  * gst_caps_new_empty_simple:
  * @media_type: the media type of the structure
@@ -317,6 +354,80 @@ gst_caps_new_empty_simple (const char *media_type)
   return caps;
 }
 
+/**
+ * gst_caps_new_static_str_empty_simple:
+ * @media_type: the media type of the structure
+ *
+ * Creates a new #GstCaps that contains one #GstStructure with name
+ * @media_type.
+ *
+ * @media_type needs to be valid for the remaining lifetime of the process, e.g.
+ * has to be a static string.
+ *
+ * Returns: (transfer full): the new #GstCaps
+ *
+ * Since: 1.26
+ */
+GstCaps *
+gst_caps_new_static_str_empty_simple (const char *media_type)
+{
+  GstCaps *caps;
+  GstStructure *structure;
+
+  caps = gst_caps_new_empty ();
+  if (strcmp ("ANY", media_type) == 0) {
+    g_warning
+        ("media_type should not be ANY. Please consider using `gst_caps_new_any` or `gst_caps_from_string`.");
+  }
+  if (strcmp ("", media_type) == 0 || strcmp ("EMPTY", media_type) == 0
+      || strcmp ("NONE", media_type) == 0) {
+    g_warning
+        ("media_type should not be `%s`. Please consider using `gst_caps_new_empty` or `gst_caps_from_string`.",
+        media_type);
+  }
+  structure = gst_structure_new_static_str_empty (media_type);
+  if (structure)
+    gst_caps_append_structure_unchecked (caps, structure, NULL);
+
+  return caps;
+}
+
+/**
+ * gst_caps_new_id_str_simple:
+ * @media_type: the media type of the structure
+ * @fieldname: first field to set
+ * @...: additional arguments
+ *
+ * Creates a new #GstCaps that contains one #GstStructure.  The
+ * structure is defined by the arguments, which have the same format
+ * as gst_structure_new().
+ *
+ * Returns: (transfer full): the new #GstCaps
+ *
+ * Since: 1.26
+ */
+GstCaps *
+gst_caps_new_id_str_simple (const GstIdStr * media_type,
+    const GstIdStr * fieldname, ...)
+{
+  GstCaps *caps;
+  GstStructure *structure;
+  va_list var_args;
+
+  caps = gst_caps_new_empty ();
+
+  va_start (var_args, fieldname);
+  structure = gst_structure_new_id_str_valist (media_type, fieldname, var_args);
+  va_end (var_args);
+
+  if (structure)
+    gst_caps_append_structure_unchecked (caps, structure, NULL);
+  else
+    gst_caps_replace (&caps, NULL);
+
+  return caps;
+}
+
 /**
  * gst_caps_new_simple:
  * @media_type: the media type of the structure
@@ -350,6 +461,46 @@ gst_caps_new_simple (const char *media_type, const char *fieldname, ...)
   return caps;
 }
 
+/**
+ * gst_caps_new_static_str_simple:
+ * @media_type: the media type of the structure
+ * @fieldname: first field to set
+ * @...: additional arguments
+ *
+ * Creates a new #GstCaps that contains one #GstStructure.  The
+ * structure is defined by the arguments, which have the same format
+ * as gst_structure_new().
+ *
+ * @media_type, @fieldname and all other fieldnames need to be valid for the
+ * remaining lifetime of the process, e.g. have to be static strings.
+ *
+ * Returns: (transfer full): the new #GstCaps
+ *
+ * Since: 1.26
+ */
+GstCaps *
+gst_caps_new_static_str_simple (const char *media_type, const char *fieldname,
+    ...)
+{
+  GstCaps *caps;
+  GstStructure *structure;
+  va_list var_args;
+
+  caps = gst_caps_new_empty ();
+
+  va_start (var_args, fieldname);
+  structure =
+      gst_structure_new_static_str_valist (media_type, fieldname, var_args);
+  va_end (var_args);
+
+  if (structure)
+    gst_caps_append_structure_unchecked (caps, structure, NULL);
+  else
+    gst_caps_replace (&caps, NULL);
+
+  return caps;
+}
+
 /**
  * gst_caps_new_full:
  * @struct1: the first structure to add
@@ -1092,6 +1243,36 @@ gst_caps_truncate (GstCaps * caps)
   return caps;
 }
 
+/**
+ * gst_caps_id_str_set_value:
+ * @caps: a writable caps
+ * @field: name of the field to set
+ * @value: value to set the field to
+ *
+ * Sets the given @field on all structures of @caps to the given @value.
+ * This is a convenience function for calling gst_structure_set_value() on
+ * all structures of @caps.
+ *
+ * Since: 1.26
+ **/
+void
+gst_caps_id_str_set_value (GstCaps * caps, const GstIdStr * field,
+    const GValue * value)
+{
+  guint i, len;
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+  g_return_if_fail (field != NULL);
+  g_return_if_fail (G_IS_VALUE (value));
+
+  len = GST_CAPS_LEN (caps);
+  for (i = 0; i < len; i++) {
+    GstStructure *structure = gst_caps_get_structure_unchecked (caps, i);
+    gst_structure_id_str_set_value (structure, field, value);
+  }
+}
+
 /**
  * gst_caps_set_value:
  * @caps: a writable caps
@@ -1119,6 +1300,80 @@ gst_caps_set_value (GstCaps * caps, const char *field, const GValue * value)
   }
 }
 
+/**
+ * gst_caps_set_value_static_str:
+ * @caps: a writable caps
+ * @field: name of the field to set
+ * @value: value to set the field to
+ *
+ * Sets the given @field on all structures of @caps to the given @value.
+ * This is a convenience function for calling gst_structure_set_value() on
+ * all structures of @caps.
+ *
+ * @field needs to be valid for the remaining lifetime of the process, e.g.
+ * has to be a static string.
+ *
+ * Since: 1.26
+ **/
+void
+gst_caps_set_value_static_str (GstCaps * caps, const char *field,
+    const GValue * value)
+{
+  guint i, len;
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+  g_return_if_fail (field != NULL);
+  g_return_if_fail (G_IS_VALUE (value));
+
+  len = GST_CAPS_LEN (caps);
+  for (i = 0; i < len; i++) {
+    GstStructure *structure = gst_caps_get_structure_unchecked (caps, i);
+    gst_structure_set_value_static_str (structure, field, value);
+  }
+}
+
+/**
+ * gst_caps_id_str_set_simple_valist:
+ * @caps: the #GstCaps to set
+ * @field: first field to set
+ * @varargs: additional parameters
+ *
+ * Sets fields in a #GstCaps.  The arguments must be passed in the same
+ * manner as gst_structure_id_str_set(), and be %NULL-terminated.
+ *
+ * Since: 1.26
+ */
+void
+gst_caps_id_str_set_simple_valist (GstCaps * caps, const GstIdStr * field,
+    va_list varargs)
+{
+  GValue value = { 0, };
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+
+  while (field) {
+    GType type;
+    char *err;
+
+    type = va_arg (varargs, GType);
+
+    G_VALUE_COLLECT_INIT (&value, type, varargs, 0, &err);
+    if (G_UNLIKELY (err)) {
+      g_critical ("%s", err);
+      g_free (err);
+      return;
+    }
+
+    gst_caps_id_str_set_value (caps, field, &value);
+
+    g_value_unset (&value);
+
+    field = va_arg (varargs, const GstIdStr *);
+  }
+}
+
 /**
  * gst_caps_set_simple_valist:
  * @caps: the #GstCaps to set
@@ -1157,6 +1412,74 @@ gst_caps_set_simple_valist (GstCaps * caps, const char *field, va_list varargs)
   }
 }
 
+/**
+ * gst_caps_set_simple_static_str_valist:
+ * @caps: the #GstCaps to set
+ * @field: first field to set
+ * @varargs: additional parameters
+ *
+ * Sets fields in a #GstCaps.  The arguments must be passed in the same
+ * manner as gst_structure_set(), and be %NULL-terminated.
+ *
+ * @field and all other field names need to be valid for the remaining lifetime
+ * of the process, e.g. have to be static strings.
+ *
+ * Since: 1.26
+ */
+void
+gst_caps_set_simple_static_str_valist (GstCaps * caps, const char *field,
+    va_list varargs)
+{
+  GValue value = { 0, };
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+
+  while (field) {
+    GType type;
+    char *err;
+
+    type = va_arg (varargs, GType);
+
+    G_VALUE_COLLECT_INIT (&value, type, varargs, 0, &err);
+    if (G_UNLIKELY (err)) {
+      g_critical ("%s", err);
+      g_free (err);
+      return;
+    }
+
+    gst_caps_set_value_static_str (caps, field, &value);
+
+    g_value_unset (&value);
+
+    field = va_arg (varargs, const gchar *);
+  }
+}
+
+/**
+ * gst_caps_id_str_set_simple:
+ * @caps: the #GstCaps to set
+ * @field: first field to set
+ * @...: additional parameters
+ *
+ * Sets fields in a #GstCaps.  The arguments must be passed in the same
+ * manner as gst_structure_id_str_set(), and be %NULL-terminated.
+ *
+ * Since: 1.26
+ */
+void
+gst_caps_id_str_set_simple (GstCaps * caps, const GstIdStr * field, ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+
+  va_start (var_args, field);
+  gst_caps_id_str_set_simple_valist (caps, field, var_args);
+  va_end (var_args);
+}
+
 /**
  * gst_caps_set_simple:
  * @caps: the #GstCaps to set
@@ -1179,6 +1502,33 @@ gst_caps_set_simple (GstCaps * caps, const char *field, ...)
   va_end (var_args);
 }
 
+/**
+ * gst_caps_set_simple_static_str:
+ * @caps: the #GstCaps to set
+ * @field: first field to set
+ * @...: additional parameters
+ *
+ * Sets fields in a #GstCaps.  The arguments must be passed in the same
+ * manner as gst_structure_set(), and be %NULL-terminated.
+ *
+ * @field and all other field names need to be valid for the remaining lifetime
+ * of the process, e.g. have to be static strings.
+ *
+ * Since: 1.26
+ */
+void
+gst_caps_set_simple_static_str (GstCaps * caps, const char *field, ...)
+{
+  va_list var_args;
+
+  g_return_if_fail (GST_IS_CAPS (caps));
+  g_return_if_fail (IS_WRITABLE (caps));
+
+  va_start (var_args, field);
+  gst_caps_set_simple_static_str_valist (caps, field, var_args);
+  va_end (var_args);
+}
+
 /* tests */
 
 /**
@@ -1217,7 +1567,7 @@ gst_caps_is_empty (const GstCaps * caps)
 }
 
 static gboolean
-gst_caps_is_fixed_foreach (GQuark field_id, const GValue * value,
+gst_caps_is_fixed_foreach (const GstIdStr * field, const GValue * value,
     gpointer unused)
 {
   return gst_value_is_fixed (value);
@@ -1254,7 +1604,8 @@ gst_caps_is_fixed (const GstCaps * caps)
 
   structure = gst_caps_get_structure_unchecked (caps, 0);
 
-  return gst_structure_foreach (structure, gst_caps_is_fixed_foreach, NULL);
+  return gst_structure_foreach_id_str (structure, gst_caps_is_fixed_foreach,
+      NULL);
 }
 
 /**
@@ -1810,7 +2161,7 @@ typedef struct
 } SubtractionEntry;
 
 static gboolean
-gst_caps_structure_subtract_field (GQuark field_id, const GValue * value,
+gst_caps_structure_subtract_field (const GstIdStr * field, const GValue * value,
     gpointer user_data)
 {
   SubtractionEntry *e = user_data;
@@ -1818,7 +2169,7 @@ gst_caps_structure_subtract_field (GQuark field_id, const GValue * value,
   const GValue *other;
   GstStructure *structure;
 
-  other = gst_structure_id_get_value (e->subtract_from, field_id);
+  other = gst_structure_id_str_get_value (e->subtract_from, field);
 
   if (!other) {
     return FALSE;
@@ -1832,7 +2183,7 @@ gst_caps_structure_subtract_field (GQuark field_id, const GValue * value,
     return FALSE;
   } else {
     structure = gst_structure_copy (e->subtract_from);
-    gst_structure_id_take_value (structure, field_id, &subtraction);
+    gst_structure_id_str_take_value (structure, field, &subtraction);
     e->put_into = g_slist_prepend (e->put_into, structure);
     return TRUE;
   }
@@ -1847,7 +2198,7 @@ gst_caps_structure_subtract (GSList ** into, const GstStructure * minuend,
 
   e.subtract_from = minuend;
   e.put_into = NULL;
-  ret = gst_structure_foreach ((GstStructure *) subtrahend,
+  ret = gst_structure_foreach_id_str ((GstStructure *) subtrahend,
       gst_caps_structure_subtract_field, &e);
 
   if (ret) {
@@ -1927,7 +2278,7 @@ gst_caps_subtract (GstCaps * minuend, GstCaps * subtrahend)
       /* Same reason as above for ANY caps */
       g_return_val_if_fail (!gst_caps_features_is_any (min_f), NULL);
 
-      if (gst_structure_get_name_id (min) == gst_structure_get_name_id (sub) &&
+      if (gst_structure_has_name (min, gst_structure_get_name (sub)) &&
           gst_caps_features_is_equal (min_f, sub_f)) {
         GSList *list;
 
@@ -1972,7 +2323,8 @@ typedef struct _NormalizeForeach
 } NormalizeForeach;
 
 static gboolean
-gst_caps_normalize_foreach (GQuark field_id, const GValue * value, gpointer ptr)
+gst_caps_normalize_foreach (const GstIdStr * field, const GValue * value,
+    gpointer ptr)
 {
   NormalizeForeach *nf = (NormalizeForeach *) ptr;
   GValue val = { 0 };
@@ -1985,13 +2337,13 @@ gst_caps_normalize_foreach (GQuark field_id, const GValue * value, gpointer ptr)
       const GValue *v = gst_value_list_get_value (value, i);
       GstStructure *structure = gst_structure_copy (nf->structure);
 
-      gst_structure_id_set_value (structure, field_id, v);
+      gst_structure_id_str_set_value (structure, field, v);
       gst_caps_append_structure_unchecked (nf->caps, structure,
           gst_caps_features_copy_conditional (nf->features));
     }
 
     gst_value_init_and_copy (&val, gst_value_list_get_value (value, 0));
-    gst_structure_id_take_value (nf->structure, field_id, &val);
+    gst_structure_id_str_take_value (nf->structure, field, &val);
     return FALSE;
   }
 
@@ -2026,7 +2378,7 @@ gst_caps_normalize (GstCaps * caps)
   for (i = 0; i < gst_caps_get_size (nf.caps); i++) {
     nf.structure = gst_caps_get_structure_unchecked (nf.caps, i);
     nf.features = gst_caps_get_features_unchecked (nf.caps, i);
-    while (!gst_structure_foreach (nf.structure,
+    while (!gst_structure_foreach_id_str (nf.structure,
             gst_caps_normalize_foreach, &nf));
   }
 
@@ -2053,20 +2405,20 @@ gst_caps_compare_structures (gconstpointer one, gconstpointer two)
 
 typedef struct
 {
-  GQuark name;
+  GstIdStr name;
   GValue value;
   GstStructure *compare;
 } UnionField;
 
 static gboolean
-gst_caps_structure_figure_out_union (GQuark field_id, const GValue * value,
-    gpointer user_data)
+gst_caps_structure_figure_out_union (const GstIdStr * field,
+    const GValue * value, gpointer user_data)
 {
   UnionField *u = user_data;
-  const GValue *val = gst_structure_id_get_value (u->compare, field_id);
+  const GValue *val = gst_structure_id_str_get_value (u->compare, field);
 
   if (!val) {
-    if (u->name)
+    if (G_VALUE_TYPE (&u->value) != G_TYPE_INVALID)
       g_value_unset (&u->value);
     return FALSE;
   }
@@ -2074,12 +2426,12 @@ gst_caps_structure_figure_out_union (GQuark field_id, const GValue * value,
   if (gst_value_compare (val, value) == GST_VALUE_EQUAL)
     return TRUE;
 
-  if (u->name) {
+  if (G_VALUE_TYPE (&u->value) != G_TYPE_INVALID) {
     g_value_unset (&u->value);
     return FALSE;
   }
 
-  u->name = field_id;
+  gst_id_str_copy_into (&u->name, field);
   gst_value_union (&u->value, val, value);
 
   return TRUE;
@@ -2090,7 +2442,7 @@ gst_caps_structure_simplify (GstStructure ** result,
     GstStructure * simplify, GstStructure * compare)
 {
   GSList *list;
-  UnionField field = { 0, {0,}, NULL };
+  UnionField field = { GST_ID_STR_INIT, G_VALUE_INIT, NULL };
 
   /* try to subtract to get a real subset */
   if (gst_caps_structure_subtract (&list, simplify, compare)) {
@@ -2110,7 +2462,7 @@ gst_caps_structure_simplify (GstStructure ** result,
 
   /* try to union both structs */
   field.compare = compare;
-  if (gst_structure_foreach (simplify,
+  if (gst_structure_foreach_id_str (simplify,
           gst_caps_structure_figure_out_union, &field)) {
     gboolean ret = FALSE;
 
@@ -2118,7 +2470,7 @@ gst_caps_structure_simplify (GstStructure ** result,
      * but at most one field: field.name */
     if (G_IS_VALUE (&field.value)) {
       if (gst_structure_n_fields (simplify) == gst_structure_n_fields (compare)) {
-        gst_structure_id_take_value (compare, field.name, &field.value);
+        gst_structure_id_str_take_value (compare, &field.name, &field.value);
         *result = NULL;
         ret = TRUE;
       } else {
@@ -2204,8 +2556,7 @@ gst_caps_simplify (GstCaps * caps)
     compare_f = gst_caps_get_features_unchecked (caps, start);
     if (!compare_f)
       compare_f = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
-    if (gst_structure_get_name_id (simplify) !=
-        gst_structure_get_name_id (compare) ||
+    if (!gst_structure_has_name (simplify, gst_structure_get_name (compare)) ||
         !gst_caps_features_is_equal (simplify_f, compare_f))
       start = i;
     for (j = start; j >= 0; j--) {
@@ -2215,9 +2566,8 @@ gst_caps_simplify (GstCaps * caps)
       compare_f = gst_caps_get_features_unchecked (caps, j);
       if (!compare_f)
         compare_f = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
-      if (gst_structure_get_name_id (simplify) !=
-          gst_structure_get_name_id (compare) ||
-          !gst_caps_features_is_equal (simplify_f, compare_f)) {
+      if (!gst_structure_has_name (simplify, gst_structure_get_name (compare))
+          || !gst_caps_features_is_equal (simplify_f, compare_f)) {
         break;
       }
       if (gst_caps_structure_simplify (&result, simplify, compare)) {
@@ -2552,8 +2902,8 @@ gst_caps_transform_to_string (const GValue * src_value, GValue * dest_value)
 /**
  * gst_caps_foreach:
  * @caps: a #GstCaps
- * @func: (scope call): a function to call for each field
- * @user_data: (closure): private data
+ * @func: (scope call) (closure user_data): a function to call for each field
+ * @user_data: private data
  *
  * Calls the provided function once for each structure and caps feature in the
  * #GstCaps. The function must not modify the fields.
@@ -2593,8 +2943,8 @@ gst_caps_foreach (const GstCaps * caps, GstCapsForeachFunc func,
 /**
  * gst_caps_map_in_place:
  * @caps: a #GstCaps
- * @func: (scope call): a function to call for each field
- * @user_data: (closure): private data
+ * @func: (scope call) (closure user_data): a function to call for each field
+ * @user_data: private data
  *
  * Calls the provided function once for each structure and caps feature in the
  * #GstCaps. In contrast to gst_caps_foreach(), the function may modify but not
@@ -2641,8 +2991,8 @@ gst_caps_map_in_place (GstCaps * caps, GstCapsMapFunc func, gpointer user_data)
 /**
  * gst_caps_filter_and_map_in_place:
  * @caps: a #GstCaps
- * @func: (scope call): a function to call for each field
- * @user_data: (closure): private data
+ * @func: (scope call) (closure user_data): a function to call for each field
+ * @user_data: private data
  *
  * Calls the provided function once for each structure and caps feature in the
  * #GstCaps. In contrast to gst_caps_foreach(), the function may modify the
