@@ -47,7 +47,6 @@
 #include "gstdeviceprovider.h"
 
 #include "gstelementmetadata.h"
-#include "gstquark.h"
 
 struct _GstDeviceProviderPrivate
 {
@@ -55,7 +54,7 @@ struct _GstDeviceProviderPrivate
 
   GMutex start_lock;
 
-  gboolean started_count;
+  gint started_count;
 
   GList *hidden_providers;
 };
@@ -84,7 +83,7 @@ static gint private_offset = 0;
 GType
 gst_device_provider_get_type (void)
 {
-  static volatile gsize gst_device_provider_type = 0;
+  static gsize gst_device_provider_type = 0;
 
   if (g_once_init_enter (&gst_device_provider_type)) {
     GType _type;
@@ -151,13 +150,11 @@ gst_device_provider_class_init (GstDeviceProviderClass * klass)
 
   gst_device_provider_signals[PROVIDER_HIDDEN] =
       g_signal_new ("provider-hidden", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_FIRST, 0, NULL,
-      NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1, G_TYPE_STRING);
+      G_SIGNAL_RUN_FIRST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_STRING);
 
   gst_device_provider_signals[PROVIDER_UNHIDDEN] =
       g_signal_new ("provider-unhidden", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_FIRST, 0, NULL,
-      NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1, G_TYPE_STRING);
+      G_SIGNAL_RUN_FIRST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_STRING);
 }
 
 static void
@@ -167,6 +164,8 @@ gst_device_provider_init (GstDeviceProvider * provider)
 
   g_mutex_init (&provider->priv->start_lock);
 
+  provider->priv->started_count = 0;
+
   provider->priv->bus = gst_bus_new ();
   gst_bus_set_flushing (provider->priv->bus, TRUE);
 }
@@ -180,8 +179,8 @@ gst_device_provider_dispose (GObject * object)
   gst_object_replace ((GstObject **) & provider->priv->bus, NULL);
 
   GST_OBJECT_LOCK (provider);
-  g_list_free_full (provider->devices, (GDestroyNotify) gst_object_unparent);
-  provider->devices = NULL;
+  g_clear_list (&provider->devices, (GDestroyNotify) gst_object_unparent);
+  g_clear_list (&provider->priv->hidden_providers, (GDestroyNotify) g_free);
   GST_OBJECT_UNLOCK (provider);
 
   G_OBJECT_CLASS (gst_device_provider_parent_class)->dispose (object);
@@ -279,11 +278,11 @@ gst_device_provider_class_set_metadata (GstDeviceProviderClass * klass,
   g_return_if_fail (description != NULL && *description != '\0');
   g_return_if_fail (author != NULL && *author != '\0');
 
-  gst_structure_id_set ((GstStructure *) klass->metadata,
-      GST_QUARK (ELEMENT_METADATA_LONGNAME), G_TYPE_STRING, longname,
-      GST_QUARK (ELEMENT_METADATA_KLASS), G_TYPE_STRING, classification,
-      GST_QUARK (ELEMENT_METADATA_DESCRIPTION), G_TYPE_STRING, description,
-      GST_QUARK (ELEMENT_METADATA_AUTHOR), G_TYPE_STRING, author, NULL);
+  gst_structure_set_static_str ((GstStructure *) klass->metadata,
+      GST_ELEMENT_METADATA_LONGNAME, G_TYPE_STRING, longname,
+      GST_ELEMENT_METADATA_KLASS, G_TYPE_STRING, classification,
+      GST_ELEMENT_METADATA_DESCRIPTION, G_TYPE_STRING, description,
+      GST_ELEMENT_METADATA_AUTHOR, G_TYPE_STRING, author, NULL);
 }
 
 /**
@@ -327,17 +326,17 @@ gst_device_provider_class_set_static_metadata (GstDeviceProviderClass * klass,
   g_value_init (&val, G_TYPE_STRING);
 
   g_value_set_static_string (&val, longname);
-  gst_structure_id_set_value (s, GST_QUARK (ELEMENT_METADATA_LONGNAME), &val);
+  gst_structure_set_value_static_str (s, GST_ELEMENT_METADATA_LONGNAME, &val);
 
   g_value_set_static_string (&val, classification);
-  gst_structure_id_set_value (s, GST_QUARK (ELEMENT_METADATA_KLASS), &val);
+  gst_structure_set_value_static_str (s, GST_ELEMENT_METADATA_KLASS, &val);
 
   g_value_set_static_string (&val, description);
-  gst_structure_id_set_value (s, GST_QUARK (ELEMENT_METADATA_DESCRIPTION),
+  gst_structure_set_value_static_str (s, GST_ELEMENT_METADATA_DESCRIPTION,
       &val);
 
   g_value_set_static_string (&val, author);
-  gst_structure_id_take_value (s, GST_QUARK (ELEMENT_METADATA_AUTHOR), &val);
+  gst_structure_set_value_static_str (s, GST_ELEMENT_METADATA_AUTHOR, &val);
 }
 
 /**
@@ -391,6 +390,9 @@ gst_device_provider_get_metadata (GstDeviceProvider * provider,
  * Gets a list of devices that this provider understands. This may actually
  * probe the hardware if the provider is not currently started.
  *
+ * If the provider has been started, this will returned the same #GstDevice
+ * objedcts that have been returned by the #GST_MESSAGE_DEVICE_ADDED messages.
+ *
  * Returns: (transfer full) (element-type GstDevice): a #GList of
  *   #GstDevice
  *
@@ -416,9 +418,15 @@ gst_device_provider_get_devices (GstDeviceProvider * provider)
     for (item = provider->devices; item; item = item->next)
       devices = g_list_prepend (devices, gst_object_ref (item->data));
     GST_OBJECT_UNLOCK (provider);
-  } else if (klass->probe)
+  } else if (klass->probe) {
+
     devices = klass->probe (provider);
 
+    for (item = devices; item; item = item->next)
+      if (g_object_is_floating (item->data))
+        g_object_ref_sink (item->data);
+  }
+
   g_mutex_unlock (&provider->priv->start_lock);
 
   return devices;
@@ -437,6 +445,10 @@ gst_device_provider_get_devices (GstDeviceProvider * provider)
  * user of the object, gst_device_provider_stop() needs to be called the same
  * number of times.
  *
+ * After this function has been called, gst_device_provider_get_devices() will
+ * return the same objects that have been received from the
+ * #GST_MESSAGE_DEVICE_ADDED messages and will no longer probe.
+ *
  * Returns: %TRUE if the device providering could be started
  *
  * Since: 1.4
@@ -459,12 +471,34 @@ gst_device_provider_start (GstDeviceProvider * provider)
     goto started;
   }
 
-  if (klass->start)
+  gst_bus_set_flushing (provider->priv->bus, FALSE);
+
+  if (klass->start) {
     ret = klass->start (provider);
+  } else {
+    GList *devices = NULL, *item;
+
+    devices = klass->probe (provider);
+
+    for (item = devices; item; item = item->next) {
+      GstDevice *device = GST_DEVICE (item->data);
+      gboolean was_floating = g_object_is_floating (item->data);
+
+      gst_device_provider_device_add (provider, device);
+
+      if (!was_floating)
+        g_object_unref (item->data);
+    }
+
+    g_list_free (devices);
+
+    ret = TRUE;
+  }
 
   if (ret) {
     provider->priv->started_count++;
-    gst_bus_set_flushing (provider->priv->bus, FALSE);
+  } else if (provider->priv->started_count == 0) {
+    gst_bus_set_flushing (provider->priv->bus, TRUE);
   }
 
 started:
@@ -500,8 +534,8 @@ gst_device_provider_stop (GstDeviceProvider * provider)
     if (klass->stop)
       klass->stop (provider);
     GST_OBJECT_LOCK (provider);
-    g_list_free_full (provider->devices, (GDestroyNotify) gst_object_unparent);
-    provider->devices = NULL;
+    g_clear_list (&provider->devices, (GDestroyNotify) gst_object_unparent);
+    g_clear_list (&provider->priv->hidden_providers, (GDestroyNotify) g_free);
     GST_OBJECT_UNLOCK (provider);
   } else if (provider->priv->started_count < 1) {
     g_critical
@@ -513,14 +547,13 @@ gst_device_provider_stop (GstDeviceProvider * provider)
   g_mutex_unlock (&provider->priv->start_lock);
 }
 
-
 /**
  * gst_device_provider_get_factory:
  * @provider: a #GstDeviceProvider to request the device provider factory of.
  *
  * Retrieves the factory that was used to create this device provider.
  *
- * Returns: (transfer none): the #GstDeviceProviderFactory used for
+ * Returns: (transfer none) (nullable): the #GstDeviceProviderFactory used for
  *     creating this device provider. no refcounting is needed.
  *
  * Since: 1.4
@@ -776,7 +809,7 @@ gst_device_provider_unhide_provider (GstDeviceProvider * provider,
 /**
  * gst_device_provider_device_changed:
  * @device: (transfer none): the new version of @changed_device
- * @changed_device: (transfer floating): the old version of the device that has been udpated
+ * @changed_device: (transfer floating): the old version of the device that has been updated
  *
  * This function is used when @changed_device was modified into its new form
  * @device. This will post a `DEVICE_CHANGED` message on the bus to let
@@ -822,3 +855,24 @@ gst_device_provider_device_changed (GstDeviceProvider * provider,
   gst_bus_post (provider->priv->bus, message);
   gst_object_unparent (GST_OBJECT (changed_device));
 }
+
+/**
+ * gst_device_provider_is_started:
+ * @provider: a #GstDeviceProvider
+ *
+ * This function can be used to know if the @provider was successfully started.
+ *
+ * Since: 1.20
+ */
+gboolean
+gst_device_provider_is_started (GstDeviceProvider * provider)
+{
+  gboolean started = FALSE;
+
+  g_return_val_if_fail (GST_IS_DEVICE_PROVIDER (provider), FALSE);
+
+  g_mutex_lock (&provider->priv->start_lock);
+  started = provider->priv->started_count > 0;
+  g_mutex_unlock (&provider->priv->start_lock);
+  return started;
+}
