@@ -40,6 +40,7 @@
 #include <gst/gstinfo.h>
 #include <gst/gstenumtypes.h>
 #include <gst/gstpadtemplate.h>
+#include "glib-compat-private.h"
 
 #include <gst/gstregistrychunks.h>
 
@@ -95,7 +96,7 @@ _strnlen (const gchar * str, gint maxlen)
   gint _len = _strnlen (inptr, (endptr-inptr)); \
   if (_len == -1) \
     goto error_label; \
-  outptr = g_memdup ((gconstpointer)inptr, _len + 1); \
+  outptr = g_memdup2 ((gconstpointer)inptr, _len + 1); \
   inptr += _len + 1; \
 }G_STMT_END
 
@@ -115,12 +116,9 @@ void
 _priv_gst_registry_chunk_free (GstRegistryChunk * chunk)
 {
   if (!(chunk->flags & GST_REGISTRY_CHUNK_FLAG_CONST)) {
-    if ((chunk->flags & GST_REGISTRY_CHUNK_FLAG_MALLOC))
-      g_free (chunk->data);
-    else
-      g_slice_free1 (chunk->size, chunk->data);
+    g_free (chunk->data);
   }
-  g_slice_free (GstRegistryChunk, chunk);
+  g_free (chunk);
 }
 
 /*
@@ -140,7 +138,7 @@ gst_registry_chunks_save_const_string (GList ** list, const gchar * str)
     str = "";
   }
 
-  chunk = g_slice_new (GstRegistryChunk);
+  chunk = g_new (GstRegistryChunk, 1);
   chunk->data = (gpointer) str;
   chunk->size = strlen ((gchar *) chunk->data) + 1;
   chunk->flags = GST_REGISTRY_CHUNK_FLAG_CONST;
@@ -161,10 +159,10 @@ gst_registry_chunks_save_string (GList ** list, gchar * str)
 {
   GstRegistryChunk *chunk;
 
-  chunk = g_slice_new (GstRegistryChunk);
+  chunk = g_new (GstRegistryChunk, 1);
   chunk->data = str;
   chunk->size = strlen ((gchar *) chunk->data) + 1;
-  chunk->flags = GST_REGISTRY_CHUNK_FLAG_MALLOC;
+  chunk->flags = GST_REGISTRY_CHUNK_FLAG_NONE;
   chunk->align = FALSE;
   *list = g_list_prepend (*list, chunk);
   return TRUE;
@@ -182,7 +180,7 @@ gst_registry_chunks_make_data (gpointer data, gulong size)
 {
   GstRegistryChunk *chunk;
 
-  chunk = g_slice_new (GstRegistryChunk);
+  chunk = g_new (GstRegistryChunk, 1);
   chunk->data = data;
   chunk->size = size;
   chunk->flags = GST_REGISTRY_CHUNK_FLAG_NONE;
@@ -205,7 +203,7 @@ gst_registry_chunks_save_pad_template (GList ** list,
   GstRegistryChunkPadTemplate *pt;
   GstRegistryChunk *chk;
 
-  pt = g_slice_new (GstRegistryChunkPadTemplate);
+  pt = g_new (GstRegistryChunkPadTemplate, 1);
   chk =
       gst_registry_chunks_make_data (pt, sizeof (GstRegistryChunkPadTemplate));
 
@@ -250,7 +248,7 @@ gst_registry_chunks_save_feature (GList ** list, GstPluginFeature * feature)
     /* Initialize with zeroes because of struct padding and
      * valgrind complaining about copying uninitialized memory
      */
-    ef = g_slice_new0 (GstRegistryChunkElementFactory);
+    ef = g_new0 (GstRegistryChunkElementFactory, 1);
     pf_size = sizeof (GstRegistryChunkElementFactory);
     chk = gst_registry_chunks_make_data (ef, pf_size);
     ef->npadtemplates = ef->ninterfaces = ef->nuriprotocols = 0;
@@ -310,7 +308,7 @@ gst_registry_chunks_save_feature (GList ** list, GstPluginFeature * feature)
     /* Initialize with zeroes because of struct padding and
      * valgrind complaining about copying uninitialized memory
      */
-    tff = g_slice_new0 (GstRegistryChunkTypeFindFactory);
+    tff = g_new0 (GstRegistryChunkTypeFindFactory, 1);
     pf_size = sizeof (GstRegistryChunkTypeFindFactory);
     chk = gst_registry_chunks_make_data (tff, pf_size);
     tff->nextensions = 0;
@@ -344,7 +342,7 @@ gst_registry_chunks_save_feature (GList ** list, GstPluginFeature * feature)
     /* Initialize with zeroes because of struct padding and
      * valgrind complaining about copying uninitialized memory
      */
-    tff = g_slice_new0 (GstRegistryChunkDeviceProviderFactory);
+    tff = g_new0 (GstRegistryChunkDeviceProviderFactory, 1);
     chk =
         gst_registry_chunks_make_data (tff,
         sizeof (GstRegistryChunkDeviceProviderFactory));
@@ -358,13 +356,13 @@ gst_registry_chunks_save_feature (GList ** list, GstPluginFeature * feature)
     /* Initialize with zeroes because of struct padding and
      * valgrind complaining about copying uninitialized memory
      */
-    pf = g_slice_new0 (GstRegistryChunkPluginFeature);
+    pf = g_new0 (GstRegistryChunkPluginFeature, 1);
     pf_size = sizeof (GstRegistryChunkPluginFeature);
     chk = gst_registry_chunks_make_data (pf, pf_size);
   } else if (GST_IS_DYNAMIC_TYPE_FACTORY (feature)) {
     GstRegistryChunkDynamicTypeFactory *tmp;
 
-    tmp = g_slice_new0 (GstRegistryChunkDynamicTypeFactory);
+    tmp = g_new0 (GstRegistryChunkDynamicTypeFactory, 1);
     chk =
         gst_registry_chunks_make_data (tmp,
         sizeof (GstRegistryChunkDynamicTypeFactory));
@@ -386,8 +384,8 @@ gst_registry_chunks_save_feature (GList ** list, GstPluginFeature * feature)
 
   /* Errors */
 fail:
-  g_slice_free (GstRegistryChunk, chk);
-  g_slice_free1 (pf_size, pf);
+  g_free (chk);
+  g_free (pf);
   return FALSE;
 }
 
@@ -398,7 +396,7 @@ gst_registry_chunks_save_plugin_dep (GList ** list, GstPluginDep * dep)
   GstRegistryChunk *chk;
   gchar **s;
 
-  ed = g_slice_new (GstRegistryChunkDep);
+  ed = g_new (GstRegistryChunkDep, 1);
   chk = gst_registry_chunks_make_data (ed, sizeof (GstRegistryChunkDep));
 
   ed->flags = dep->flags;
@@ -440,7 +438,7 @@ _priv_gst_registry_chunks_save_plugin (GList ** list, GstRegistry * registry,
   GList *plugin_features = NULL;
   GList *walk;
 
-  pe = g_slice_new (GstRegistryChunkPluginElement);
+  pe = g_new (GstRegistryChunkPluginElement, 1);
   chk =
       gst_registry_chunks_make_data (pe,
       sizeof (GstRegistryChunkPluginElement));
@@ -480,6 +478,14 @@ _priv_gst_registry_chunks_save_plugin (GList ** list, GstRegistry * registry,
     gst_registry_chunks_save_const_string (list, "");
   }
 
+  /* pack status info */
+  if (plugin->priv->status_info) {
+    gchar *status_str = gst_structure_to_string (plugin->priv->status_info);
+    gst_registry_chunks_save_string (list, status_str);
+  } else {
+    gst_registry_chunks_save_const_string (list, "");
+  }
+
   /* pack plugin element strings */
   gst_registry_chunks_save_const_string (list,
       (plugin->desc.release_datetime) ? plugin->desc.release_datetime : "");
@@ -501,8 +507,8 @@ _priv_gst_registry_chunks_save_plugin (GList ** list, GstRegistry * registry,
   /* Errors */
 fail:
   gst_plugin_feature_list_free (plugin_features);
-  g_slice_free (GstRegistryChunk, chk);
-  g_slice_free (GstRegistryChunkPluginElement, pe);
+  g_free (chk);
+  g_free (pe);
   return FALSE;
 }
 
@@ -526,7 +532,7 @@ gst_registry_chunks_load_pad_template (GstElementFactory * factory, gchar ** in,
       *in);
   unpack_element (*in, pt, GstRegistryChunkPadTemplate, end, fail);
 
-  template = g_slice_new (GstStaticPadTemplate);
+  template = g_new (GstStaticPadTemplate, 1);
   template->presence = pt->presence;
   template->direction = (GstPadDirection) pt->direction;
   template->static_caps.caps = NULL;
@@ -542,7 +548,7 @@ gst_registry_chunks_load_pad_template (GstElementFactory * factory, gchar ** in,
 fail:
   GST_INFO ("Reading pad template failed");
   if (template)
-    g_slice_free (GstStaticPadTemplate, template);
+    g_free (template);
   return FALSE;
 }
 
@@ -783,7 +789,7 @@ gst_registry_chunks_load_plugin_dep (GstPlugin * plugin, gchar ** in,
   GST_LOG_OBJECT (plugin, "Unpacking GstRegistryChunkDep from %p", *in);
   unpack_element (*in, d, GstRegistryChunkDep, end, fail);
 
-  dep = g_slice_new (GstPluginDep);
+  dep = g_new (GstPluginDep, 1);
 
   dep->env_hash = d->env_hash;
   dep->stat_hash = d->stat_hash;
@@ -828,6 +834,7 @@ _priv_gst_registry_chunks_load_plugin (GstRegistry * registry, gchar ** in,
   gchar *start = *in;
 #endif
   GstRegistryChunkPluginElement *pe;
+  const gchar *status_str = NULL;
   const gchar *cache_str = NULL;
   GstPlugin *plugin = NULL;
   guint i, n;
@@ -868,6 +875,11 @@ _priv_gst_registry_chunks_load_plugin (GstRegistry * registry, gchar ** in,
   if (plugin->desc.release_datetime[0] == '\0')
     plugin->desc.release_datetime = NULL;
 
+  /* unpack status info */
+  unpack_string_nocopy (*in, status_str, end, fail);
+  if (status_str != NULL && *status_str != '\0')
+    plugin->priv->status_info = gst_structure_from_string (status_str, NULL);
+
   /* unpack cache data */
   unpack_string_nocopy (*in, cache_str, end, fail);
   if (cache_str != NULL && *cache_str != '\0')
@@ -925,7 +937,7 @@ _priv_gst_registry_chunks_save_global_header (GList ** list,
   GstRegistryChunkGlobalHeader *hdr;
   GstRegistryChunk *chk;
 
-  hdr = g_slice_new (GstRegistryChunkGlobalHeader);
+  hdr = g_new (GstRegistryChunkGlobalHeader, 1);
   chk = gst_registry_chunks_make_data (hdr,
       sizeof (GstRegistryChunkGlobalHeader));
 
