@@ -25,7 +25,23 @@
 #include <gst/gst.h>
 
 #ifndef GST_BASE_API
-#define GST_BASE_API GST_EXPORT
+#ifdef BUILDING_GST_BASE
+#define GST_BASE_API GST_API_EXPORT        /* from config.h */
+#else
+#define GST_BASE_API GST_API_IMPORT
+#endif
+#endif
+
+#ifndef GST_DISABLE_DEPRECATED
+#define GST_BASE_DEPRECATED GST_BASE_API
+#define GST_BASE_DEPRECATED_FOR(f) GST_BASE_API
+#define GST_BASE_DEPRECATED_TYPE
+#define GST_BASE_DEPRECATED_TYPE_FOR(f)
+#else
+#define GST_BASE_DEPRECATED G_DEPRECATED GST_BASE_API
+#define GST_BASE_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f) GST_BASE_API
+#define GST_BASE_DEPRECATED_TYPE G_DEPRECATED
+#define GST_BASE_DEPRECATED_TYPE_FOR(f) G_DEPRECATED_FOR(f)
 #endif
 
 #endif /* __GST_BASE_PRELUDE_H__ */
